<?php

namespace Materials\Material\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Materials\Material\Http\Requests\MaterialUserWebRequest;
use Materials\Material\Interfaces\MaterialRepositoryInterface;
use Materials\Material\Models\Material;

class MaterialUserController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'web';

    /**
     * Initialize material controller.
     *
     * @param type MaterialRepositoryInterface $material
     *
     * @return type
     */
    public function __construct(MaterialRepositoryInterface $material)
    {
        $this->middleware('web');
        $this->middleware('auth:web');
        $this->setupTheme(config('theme.themes.user.theme'), config('theme.themes.user.layout'));

        $this->repository = $material;
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return Response
     */
    public function index(MaterialUserWebRequest $request)
    {
        $this->repository->pushCriteria(new \Materials\Material\Repositories\Criteria\MaterialUserCriteria());
        $materials = $this->repository->scopeQuery(function($query){
            return $query->orderBy('id','DESC');
        })->paginate();

        $this->theme->prependTitle(trans('material::material.names').' :: ');

        return $this->theme->of('material::user.material.index', compact('materials'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     * @param Material $material
     *
     * @return Response
     */
    public function show(MaterialUserWebRequest $request, Material $material)
    {
        Form::populate($material);

        return $this->theme->of('material::user.material.show', compact('material'))->render();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(MaterialUserWebRequest $request)
    {

        $material = $this->repository->newInstance([]);
        Form::populate($material);

        return $this->theme->of('material::user.material.create', compact('material'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(MaterialUserWebRequest $request)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $material = $this->repository->create($attributes);

            return redirect(trans_url('/user/material/material'))
                -> with('message', trans('messages.success.created', ['Module' => trans('material::material.name')]))
                -> with('code', 201);

        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param Request $request
     * @param Material $material
     *
     * @return Response
     */
    public function edit(MaterialUserWebRequest $request, Material $material)
    {

        Form::populate($material);

        return $this->theme->of('material::user.material.edit', compact('material'))->render();
    }

    /**
     * Update the specified resource.
     *
     * @param Request $request
     * @param Material $material
     *
     * @return Response
     */
    public function update(MaterialUserWebRequest $request, Material $material)
    {
        try {
            $this->repository->update($request->all(), $material->getRouteKey());

            return redirect(trans_url('/user/material/material'))
                ->with('message', trans('messages.success.updated', ['Module' => trans('material::material.name')]))
                ->with('code', 204);

        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Remove the specified resource.
     *
     * @param int $id
     *
     * @return Response
     */
    public function destroy(MaterialUserWebRequest $request, Material $material)
    {
        try {
            $this->repository->delete($material->getRouteKey());
            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('material::material.name')]),
                'code'     => 202,
                'redirect' => trans_url('/user/material/material/0'),
            ], 202);

        } catch (Exception $e) {

            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);

        }
    }
}
